// vim:syntax=lpc
//! @ignore
inherit Public.Parser@module@;

constant __author = "Arne Goedeke <pike@laramies.com>";
constant __version = "0.8";
constant __components = ({ "Public.pmod/Parser.pmod/JSON2.pmod/module.pmod", "Public.pmod/Parser.pmod/___JSON2.so" });
//! @endignore

class UnicodeSet {

    constant stop1 = 0xd7ff;
    constant start2 = 0xe000;
    constant stop2 = 0x10ffff;
    constant gap = start2 - stop1 - 1;

    mixed `[](mixed a, mixed|void b) {
	if (!b && zero_type(b)) {
	    if (intp(a)) switch(a) {
	    case 0 .. stop1:
		return a;
	    case stop1+1 .. stop2-gap:
		return a + gap; 
	    }
	}
	throw(({ "Out of Unicode range.\n", backtrace() }));
    }

    int _sizeof() {
	return stop1 + stop2 - stop2 + 2;
    }
}

//! Objects representing the three JSON literals @expr{null@}, @expr{false@} 
//! and @expr{true@}. They should behave as exprected in boolean context.
//! @expr{null@} can be used as the Integer value @expr{0@} in addition and
//! multiplication.
object null = Null();
object true = True();
object false = False();
object Unicode = UnicodeSet();

class Null {
    int(0..1) `!() {
	return 1;
    }

    int(0..1) `+(mixed a) {
	return a;
    }

    int `*(mixed a) {
	return 0;
    }

    mixed cast(string type) {
	if (type == "string") return "null";

	error("Cannot cast %O to %s.\n", this_program, type);
    }
}
class False {
    int(0..1) `!() {
	return 1;
    }

    mixed `~() {
	return true;
    }
    mixed cast(string type) {
	if (type == "string") return "false";

	error("Cannot cast %O to %s.\n", this_program, type);
    }
}
class True {
    int(0..1) `!() {
	return 0;
    }

    mixed `~() {
	return false;
    }
    mixed cast(string type) {
	if (type == "string") return "true";

	error("Cannot cast %O to %s.\n", this_program, type);
    }
}


constant HUMAN_READABLE = 2;
//! May be passed to @[render()] and @[render_utf8()] as a flag. Makes output 
//! readable by inserting spaces and newlines.
//! 
